/*
    module/comedi_module.h
    header file for kernel-only structures, variables, and constants

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _COMEDI_MODULE_H
#define _COMEDI_MODULE_H

#include <linux/version.h>
#include <linux/config.h>
#include <linux/kdev_t.h>
#include <config.h>

#if LINUX_VERSION_CODE < 0x020000
#error kernel versions prior to 2.0 not supported
#elseif LINUX_VERSION_CODE < 0x020100
#define LINUX_V20
#else
#define LINUX_V22
#endif

#include <comedi.h>


/* belong in comedi.h */
#define COMEDI_SUBD_UNUSED		0
#define COMEDI_SUBD_AI			1
#define COMEDI_SUBD_AO			2
#define COMEDI_SUBD_DI			3
#define COMEDI_SUBD_DO			4
#define COMEDI_SUBD_DIO			5
#define COMEDI_SUBD_COUNTER		6
#define COMEDI_SUBD_TIMER		7
#define COMEDI_SUBD_RAM			8
#define COMEDI_SUBD_CALIB		9
#define COMEDI_SUBD_PROC		10

typedef struct comedi_device_struct comedi_device;
typedef struct comedi_subdevice_struct comedi_subdevice;

typedef int comedi_devinit(comedi_device *,comedi_devconfig *);

typedef struct comedi_bufll{
	struct comedi_bufll *next;
	int use_count;
	char stuff[0];
}comedi_bufll;

typedef struct{
	comedi_bufll *first;
	comedi_bufll *last;
	int in;
	int use_count;
	int num_pages;
}comedi_buf;

typedef struct{
	comedi_buf *buf;
	comedi_bufll *bufll;
	int offset;
}comedi_bufptr;

struct comedi_subdevice_struct{
	int type;
	int n_chan;
	int subdev_flags;
	int timer_type;
	int len_chanlist;		/* length of channel/gain list, if available */

	int (*trig)(comedi_device *,comedi_trig *);
	
	void *lock;
	void *busy;
	
	int io_bits;
	
	sampl_t maxdata;		/* if maxdata==0, use list */
	sampl_t *maxdata_list;		/* list is channel specific */
	
	unsigned int flags;
	unsigned int *flag_list;

	unsigned int n_range;
	unsigned int range_type;
	unsigned int *range_type_list;
	
	unsigned int *range_list;	/* is this necessary? */
	
	void *private;
};

struct comedi_device_struct{
	int use_count;
	void *private;
	kdev_t minor;
	int (*rem)(comedi_device *);
	char *driver_name;
	char *board_name;
	int board;

	int n_subdevices;
	comedi_subdevice *subdevices;
	int options[COMEDI_NDEVCONFOPTS];

	/* dumb */
	int iobase;
	int iosize;
	int irq;

	/* why? */
	struct wait_queue *rdwait;

	/* still needed? */
	comedi_buf rdbuf;
	void (*poll)(comedi_device *);
	int polling;

};



extern comedi_device *comedi_devices;
extern comedi_devinit *comedi_devinits[];
extern const int comedi_maxinits;

/*
 * function prototypes
 */

int comedi_initbuf(comedi_buf *buf);
void comedi_freebuf(comedi_buf *buf);
void copy_to_buffer(comedi_buf *buf,void *src,int n);
int copy_from_buffer(comedi_bufptr *ptr,void *dest,int n);
comedi_bufptr *comedi_hookbuf(comedi_buf *buf);
void comedi_unhookbuf(comedi_bufptr *it);
void report_sample(comedi_device *dev,sampl_t s);
void comedi_error(comedi_device *dev,const char *s);

/*
int init_param(void);
int cleanup_param(void);
int addparam(comedi_iparam **list,int pnum,int flags,int pval);
int changeparam(comedi_iparam **list,int pnum,int flags,int pval);
int delparamlist(comedi_iparam *list);
int getparamval(comedi_iparam *list,int pnum);
*/

void init_polling(void);
void cleanup_polling(void);
void start_polling(comedi_device *);
void stop_polling(comedi_device *);

void comedi_proc_init(void);
void comedi_proc_cleanup(void);

/* range types */
/* these *must* agree with lib/range.c */

#define RANGE_unipolar10		1
#define RANGE_unipolar5			2
#define RANGE_digital			RANGE_unipolar5
#define RANGE_bipolar10			3
#define RANGE_bipolar5			4
#define RANGE_dt282x_ai_lo_bipolar	5
#define RANGE_dt282x_ai_lo_unipolar	6
#define RANGE_dt282x_ai_hi_bipolar	7
#define RANGE_dt282x_ai_hi_unipolar	8
#define RANGE_ni_E_ai			9
#define RANGE_unipolar2_5		10
#define RANGE_bipolar2_5		11
#define RANGE_dt282x_ai_5_bipolar	12
#define RANGE_dt282x_ai_5_unipolar	13
#define RANGE_pcl711b_ai		14
#define RANGE_ni_E_ai_limited		15
#define RANGE_ni_E_ai_limited14		16
#define RANGE_acl8112hg_ai              17
#define RANGE_acl8112dg_ai              18
#define RANGE_rti800_ai_10_bipolar	19
#define RANGE_rti800_ai_5_bipolar	20
#define RANGE_rti800_ai_10_unipolar	21
#define RANGE_rti800_ao_10_bipolar		RANGE_bipolar10
#define RANGE_rti800_ao_10_unipolar		RANGE_unipolar10
#define RANGE_rti802_ao_10_bipolar		RANGE_bipolar10
#define RANGE_rti802_ao_10_unipolar		RANGE_unipolar10


/* timer types */
/* these *must* agree with lib/timer.c */

#define TIMER_dt282x			1
#define TIMER_dt2814			2
#define TIMER_atmio			3
#define TIMER_acl8112			4


/* kernel compatibility */

#if LINUX_V20

#define ioremap vremap
#define iounmap vfree

#endif


#if LINUX_V20
/* actually LINUX_VERSION_CODE < 0x020155 */

#define LINUX_old_PCI_compatibility

#endif


#endif /* _COMEDI_MODULE_H */




